#pragma once
#include <clippy/config.hpp>

#include <utils/lock_file.hpp>

#include <vector>
#include <filesystem>
#include <optional>

struct Project {
  std::strong_ordering operator<=>(const Project&) const = default;

  Config GetConfig() { return Config{configuration_file, root_project}; }

  std::filesystem::path root_project;
  std::filesystem::path configuration_file;
};

class ProjectList {
 public:
  ProjectList(std::filesystem::path path) : path_(std::move(path)), lock_file_(path_) {
    Load();
  }

  Config GetNewConfig(const std::filesystem::path& config_directory);

  const std::vector<Project>& GetProjects() const { return projects_; }

  std::optional<Project> GetCurrentProject();

 private:
  void Load();
  void LoadWithouLock();

 private:
  std::filesystem::path path_;
  utils::filesystem::LockFile lock_file_;

  std::vector<Project> projects_;
};
